package mvd.jester.simulator.internals.schmidmottok;

import java.util.Optional;
import mvd.jester.simulator.internals.JobContextInterface;
import mvd.jester.simulator.internals.ProcessorContext;
import mvd.jester.simulator.internals.TaskContextInterface;

/**
 * Job
 * 
 * @param <Job>
 */
public class JobContext implements JobContextInterface {

    private final TaskContext taskContext;
    private final SegmentContext segmentContext;
    private final long wcet;
    private Optional<ProcessorContext> currentProcessor;
    private Optional<TaskletContext> currentTasklet;
    private long executionTime;


    public JobContext(TaskContext taskContext, SegmentContext segmentContext) {
        this.currentProcessor = Optional.empty();
        this.currentTasklet = Optional.empty();
        this.taskContext = taskContext;
        this.segmentContext = segmentContext;
        this.wcet = segmentContext.getSegment().getJobWcet();
        this.executionTime = wcet;
    }

    public Optional<TaskContextInterface> updateExecution(long time) {
        if (!currentTasklet.isPresent()) {
            currentTasklet = segmentContext.getNextTasklet();
            // if (currentTasklet.isPresent()) {
            // System.out.println("Time " + time + ": Job " + this + " started executing tasklet "
            // + currentTasklet.get() + " on Processor " + currentProcessor.get());
            // }
        }

        if (currentTasklet.isPresent()) {
            currentTasklet.get().setCurrentJob(this);
            return currentTasklet.get().updateExecution();
        } else {
            currentProcessor.get().setJob(null);
            currentProcessor = Optional.empty();
            return Optional.empty();
        }
    }

    public boolean checkExecutionTime() {
        return executionTime > 0;
    }

    /**
     * @return the wcet
     */
    public long getWcet() {
        return wcet;
    }

    /**
     * @param processor the currentProcessor to set
     */
    public void setCurrentProcessor(ProcessorContext processor) {
        this.currentProcessor = Optional.ofNullable(processor);
    }

    /**
     * @return the currentProcessor
     */
    public Optional<ProcessorContext> getCurrentProcessor() {
        return currentProcessor;
    }

    /**
     * @param currentTasklet the currentTasklet to set
     */
    public void setCurrentTasklet(TaskletContext currentTasklet) {
        this.currentTasklet = Optional.ofNullable(currentTasklet);
    }

    /**
     * @return the segmentContext
     */
    public SegmentContext getSegmentContext() {
        return segmentContext;
    }


    /**
     * @return the taskContext
     */
    public TaskContext getTaskContext() {
        return taskContext;
    }

    @Override
    public String toString() {
        return "(of task=" + taskContext + ")";
    }

}
