package mvd.jester.simulator.internals.schmidmottok;

import java.util.Optional;
import mvd.jester.simulator.internals.TaskContextInterface;

/**
 * Tasklet
 */
public class TaskletContext {

    private final TaskContext taskContext;
    private Optional<JobContext> currentJob;
    private final long wcet;
    private long executionTime;


    public TaskletContext(TaskContext taskContext, SegmentContext segment) {
        this.taskContext = taskContext;
        this.wcet = segment.getSegment().getTaskletWcet();
        this.executionTime = wcet;
        currentJob = Optional.empty();
    }


    /**
     * @return the currentJob
     */
    public Optional<JobContext> getCurrentJob() {
        return currentJob;
    }

    public void setCurrentJob(JobContext jobContext) {
        currentJob = Optional.ofNullable(jobContext);
    }

    public Optional<TaskContextInterface> updateExecution() {
        executionTime--;

        if (executionTime == 0) {
            currentJob.get().setCurrentTasklet(null);
            currentJob = Optional.empty();
            return taskContext.acceptNotification();
        }
        return Optional.empty();
    }

    public boolean checkExecutionTime() {
        return executionTime > 0;
    }

    @Override
    public String toString() {
        return "(wcet=" + wcet + ", task=" + taskContext + ")";
    }
}
